/* ######################################################################### */
/* ##                                                                     ## */
/* ##  D B T U T O O L S / D B 2 U E X T 2 . C                            ## */
/* ##                                                                     ## */
/* ## ------------------------------------------------------------------- ## */
/* ##                                                                     ## */
/* ##  Job .......: Implements the level one userexit which               ## */
/* ##               is directly called by IBM's DB2 UDB.                  ## */
/* ##                                                                     ## */
/* ## ------------------------------------------------------------------- ## */
/* ##                                                                     ## */
/* ##  Copyright (C) 2003  Daniel Scheibli                                ## */
/* ##                                                                     ## */
/* ##  This program is free software; you can redistribute it and/or      ## */
/* ##  modify it under the terms of the GNU General Public License        ## */
/* ##  as published by the Free Software Foundation; either version 2     ## */
/* ##  of the License, or (at your option) any later version.             ## */
/* ##                                                                     ## */
/* ##  This program is distributed in the hope that it will be useful,    ## */
/* ##  but WITHOUT ANY WARRANTY; without even the implied warranty of     ## */
/* ##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      ## */
/* ##  GNU General Public License for more details.                       ## */
/* ##                                                                     ## */
/* ##  You should have received a copy of the GNU General Public License  ## */
/* ##  along with this program; if not, write to the Free Software        ## */
/* ##  Foundation, Inc., 59 Temple Place - Suite 330, Boston,             ## */
/* ##  MA  02111-1307, USA.                                               ## */
/* ##                                                                     ## */
/* ## ------------------------------------------------------------------- ## */
/* ##                                                                     ## */
/* ##  Remark ....: <none>                                                ## */
/* ##                                                                     ## */
/* ## ------------------------------------------------------------------- ## */
/* ##                                                                     ## */
/* ##  Author ....: Daniel Scheibli <daniel.scheibli@edelbyte.org>        ## */
/* ##  Date ......: 2003-01-02                                            ## */
/* ##  Changes ...: ----- db2uext2.c V00.99/R020 ------------------------ ## */
/* ##               <none>                                                ## */
/* ##                                                                     ## */
/* ######################################################################### */



/* ##### Presettings */
/* ------------------------------------------------------------------------- */
#include "libdbtubase.h"
/* ------------------------------------------------------------------------- */
struct ArgsCommon
{
 char *pcRequest;    /* ##### "-RQ" / Request (Archive or Retrieve) */
 char *pcDbName;     /* ##### "-DB" / Database Name */
 char *pcNodeName;   /* ##### "-NN" / Node Name */
 char *pcLogPath;    /* ##### "-LP" / Log Path */
 char *pcLogName;    /* ##### "-LN" / Log Name */ 
};
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
struct GlobalStruct
{
 int                iMainArgc;        /* ##### Contains the number of arguments passed to main() */ 
 void              *pvMainArgv;       /* ##### Pointer to the arguments vector passed to main()  */
 struct ArgsCommon  sArgsCommon;      /* ##### Structure containing the pointers to the arguments (without their indicators) */
 FILE              *phErrorLogFile;   /* ##### Pointer to the error log file handler */
 int                iProgRc;          /* ##### Return code of the program */
};
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
struct GlobalStruct sGlobalStruct;
/* ------------------------------------------------------------------------- */
void ErrorLogFileClose( void );
void ErrorLogFileOpen( void );
void ErrorLogFileWriteMessage( const char *pcFormat, ... );
void ErrorLogFileWriteMessageFooter( void );
void ErrorLogFileWriteMessageHeader( char *pcCodeFileName,
                                     int   iCodeFileLine );
void ParseArguments( void );
void ProgramActionArchive( void );
void ProgramActionRetrieve( void );
void SignalHandler( int iSignalNumber );
/* ------------------------------------------------------------------------- */
#define ERRORLOGFILEWRITEMESSAGEHEADER()   ErrorLogFileWriteMessageHeader( __FILE__, __LINE__ )
/* ------------------------------------------------------------------------- */





/* ######################################################################### */
/* ##                                                                     ## */
/* ##   < M A I N >                                                       ## */
/* ##                                                                     ## */
/* ######################################################################### */
int main( int argc, char *argv[] )
{
 /* ##### Presettings */
 /* ------------------------------------------------------------------------ */
 memset( &sGlobalStruct, EOS, sizeof(struct GlobalStruct) );
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -- */
 sGlobalStruct.iMainArgc  = argc;
 sGlobalStruct.pvMainArgv = (void *)argv; 
 sGlobalStruct.iProgRc    = DBTUTOOLS_RC_USEREXIT_OK;
 /* ------------------------------------------------------------------------ */



 /* ##### "Prevent nested IF's"-Loop */
 /* ------------------------------------------------------------------------ */
 for(;;)
 {
  /* ##### Process the given parameters */
  /* ----------------------------------------------------------------------- */
  if( argc == 1 )
  {
   fprintf( STDERR, "%s", NEWLINE );
   fprintf( STDERR, "(compiled on %s at %s, %s with %s)%s", "hal", __DATE__, __TIME__, DBTUTOOLS_C_COMPILER, NEWLINE );
   fprintf( STDERR, "%s", NEWLINE );
   fprintf( STDERR, "Important compiler macros:%s", NEWLINE );
   fprintf( STDERR, " - DBTUTOOLS_USEREXIT_DB2_INSTANCE ....: >%s<%s", DBTUTOOLS_USEREXIT_DB2_INSTANCE, NEWLINE );
   fprintf( STDERR, " - DBTUTOOLS_USEREXIT_OPT_TREE_ROOT ...: >%s<%s", DBTUTOOLS_USEREXIT_OPT_TREE_ROOT, NEWLINE );
   fprintf( STDERR, " - FILE_SEPARATOR .....................: %c%s",   FILE_SEPARATOR, NEWLINE );
   fprintf( STDERR, "%s", NEWLINE );
   sGlobalStruct.iProgRc = DBTUTOOLS_RC_USEREXIT_INVALID_PARAMETERS;
   break;
  }
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  ParseArguments();
  /* ##### On error,... */
  if( sGlobalStruct.iProgRc != DBTUTOOLS_RC_USEREXIT_OK )
  { break; }
  /* ----------------------------------------------------------------------- */



  /* ##### Install/Establish signal handlers */
  /* ----------------------------------------------------------------------- */
#if defined(DBTUTOOLS_OS_PLATFORM_UNIX)
  if( ( signal( SIGHUP,  SignalHandler ) == SIG_ERR ) ||
      ( signal( SIGINT,  SignalHandler ) == SIG_ERR ) ||
      ( signal( SIGABRT, SignalHandler ) == SIG_ERR ) ||
      ( signal( SIGTERM, SignalHandler ) == SIG_ERR ) ||
      ( signal( SIGUSR1, SignalHandler ) == SIG_ERR ) )
#endif
#if defined(DBTUTOOLS_OS_PLATFORM_WINTEL)
  if( ( signal( SIGINT,  SignalHandler ) == SIG_ERR ) ||
      ( signal( SIGABRT, SignalHandler ) == SIG_ERR ) ||
      ( signal( SIGTERM, SignalHandler ) == SIG_ERR ) )
#endif
  {
   ERRORLOGFILEWRITEMESSAGEHEADER();
   ErrorLogFileWriteMessage( "===> ERROR: Installing the signal handlers failed!%s", NEWLINE );
   ErrorLogFileWriteMessageFooter();
   sGlobalStruct.iProgRc = DBTUTOOLS_RC_USEREXIT_ALLOC_FAILED;
   break;
  }
  /* ----------------------------------------------------------------------- */



  /* ##### Start processing based on the given request */
  /* ----------------------------------------------------------------------- */
  if( STRCMP( sGlobalStruct.sArgsCommon.pcRequest, "ARCHIVE" ) == 0 )
  { ProgramActionArchive(); }
  else
  {
   if( STRCMP( sGlobalStruct.sArgsCommon.pcRequest, "RETRIEVE" ) == 0 )
   { ProgramActionRetrieve(); }
   else
   {
    ERRORLOGFILEWRITEMESSAGEHEADER();
    ErrorLogFileWriteMessage( "===> ERROR: The parameter '-RQ' contains an invalid request identifier.%s", NEWLINE );
    ErrorLogFileWriteMessageFooter();
    sGlobalStruct.iProgRc = DBTUTOOLS_RC_USEREXIT_INVALID_PARAMETERS;
   }
  }
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  /* ##### On error,... */
  if( sGlobalStruct.iProgRc != DBTUTOOLS_RC_USEREXIT_OK )
  { break; }  
  /* ----------------------------------------------------------------------- */



/* ##### While we haven't finished development, we have to return */
/* ##### a permanent error to ensure that this will not be used!  */
/* ######################################################################### */
sGlobalStruct.iProgRc = DBTUTOOLS_RC_USEREXIT_SOFTWARE_FAILURE;
/* ######################################################################### */



  /* ##### Leave "Prevent nested IF's"-Loop */
  break;
 }
 /* ----------------------------------------------------------------------- */



 /* ##### Close error log file and end the program */
 /* ------------------------------------------------------------------------ */ 
 ErrorLogFileClose();
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -- */
 return( sGlobalStruct.iProgRc );
 /* ------------------------------------------------------------------------ */ 
}





/* ######################################################################### */
/* ##                                                                     ## */
/* ##   E R R O R L O G F I L E C L O S E ()                              ## */
/* ##                                                                     ## */
/* ######################################################################### */
void ErrorLogFileClose( void )
{
 /* ##### Close the error log files (if it is still open) */
 /* ------------------------------------------------------------------------ */
 if( sGlobalStruct.phErrorLogFile != NULL )
 { fclose( sGlobalStruct.phErrorLogFile ); }
 /* ------------------------------------------------------------------------ */
}





/* ######################################################################### */
/* ##                                                                     ## */
/* ##   E R R O R L O G F I L E O P E N ()                                ## */
/* ##                                                                     ## */
/* ######################################################################### */
void ErrorLogFileOpen( void )
{
 /* ##### Presettings */
 /* ------------------------------------------------------------------------ */
 char acFileName[DBTUTOOLS_MEMORY_SIZE_SMALL];
 int  iCallRc                                 = 0;
 /* ------------------------------------------------------------------------ */ 



 /* ##### Open the error log file (if it isn't already open) */
 /* ------------------------------------------------------------------------ */
 if( sGlobalStruct.phErrorLogFile == NULL )
 {
  /* ##### Assemble the error log file name */
  iCallRc = SNPRINTF( acFileName, DBTUTOOLS_MEMORY_SIZE_SMALL, "%s%c%s%c%s%c%s%c%s", DBTUTOOLS_USEREXIT_OPT_TREE_ROOT, FILE_SEPARATOR,
 				                               DBTUTOOLS_USEREXIT_DB2_INSTANCE, FILE_SEPARATOR,
				                               "tools",                         FILE_SEPARATOR,
				                               "log",                           FILE_SEPARATOR,
					                       "db2uext2.err" );
  /* ##### If nothing went wrong,... */
  if( !(( iCallRc == -1 ) || ( iCallRc >= DBTUTOOLS_MEMORY_SIZE_SMALL )) )
  { sGlobalStruct.phErrorLogFile = fopen( acFileName, "a" ); }    
 }
 /* ------------------------------------------------------------------------ */
}





/* ######################################################################### */
/* ##                                                                     ## */
/* ##   E R R O R L O G F I L E W R I T E M E S S A G E ()                ## */
/* ##                                                                     ## */
/* ######################################################################### */
void ErrorLogFileWriteMessage( const char *pcFormat, ... )
{
 /* ##### Presettings */
 /* ------------------------------------------------------------------------ */
 va_list hVariableArguments;
 /* ------------------------------------------------------------------------ */



 /* ##### Ensure, that the error log files is already open */
 /* ------------------------------------------------------------------------ */
 if( sGlobalStruct.phErrorLogFile == NULL )
 {
  /* ##### Open the error log file */
  ErrorLogFileOpen();
  /* ##### On error,... */
  if( sGlobalStruct.phErrorLogFile == NULL )
  { return; }
 }
 /* ------------------------------------------------------------------------ */


 
 /* ##### Write the message into the error log file */
 /* ------------------------------------------------------------------------ */
 va_start( hVariableArguments, pcFormat );
 vfprintf( sGlobalStruct.phErrorLogFile, pcFormat, hVariableArguments );
 va_end( hVariableArguments );
 /* ------------------------------------------------------------------------ */
}





/* ######################################################################### */
/* ##                                                                     ## */
/* ##   E R R O R L O G F I L E W R I T E M E S S A G E F O O T E R ()    ## */
/* ##                                                                     ## */
/* ######################################################################### */
void ErrorLogFileWriteMessageFooter( void )
{
 /* ##### Presettings */
 /* ------------------------------------------------------------------------ */
 /* ------------------------------------------------------------------------ */ 



 /* ##### Write ending status line to the error log file */
 /* ------------------------------------------------------------------------ */
 ErrorLogFileWriteMessage( "##### END #####################################################################%s", NEWLINE );
 ErrorLogFileWriteMessage( "%s", NEWLINE);
 /* ------------------------------------------------------------------------ */
}





/* ######################################################################### */
/* ##                                                                     ## */
/* ##   E R R O R L O G F I L E W R I T E M E S S A G E H E A D E R ()    ## */
/* ##                                                                     ## */
/* ######################################################################### */
void ErrorLogFileWriteMessageHeader( char *pcCodeFileName, int iCodeFileLine )
{
 /* ##### Presettings */
 /* ------------------------------------------------------------------------ */
 time_t      lSecondsSince;
 struct tm  *sTmFields;
 char      **ppcMainArgv;
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -- */ 
 int         I;
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -- */
 time( &lSecondsSince );
 sTmFields   = localtime( &lSecondsSince );
 ppcMainArgv = sGlobalStruct.pvMainArgv;
 /* ------------------------------------------------------------------------ */ 



 /* ##### Write all (maybe) usefull informations to the error log file */
 /* ------------------------------------------------------------------------ */
 ErrorLogFileWriteMessage( "##### BEGIN ###################################################################%s", NEWLINE );
 ErrorLogFileWriteMessage( "Basic informations:%s",                  NEWLINE);
 ErrorLogFileWriteMessage( " - Current time .......................: %04d-%02d-%02d %02d:%02d:%02d%s", (sTmFields->tm_year) + 1900,
                                                                     (sTmFields->tm_mon) + 1,
					 	     	             sTmFields->tm_mday,
							             sTmFields->tm_hour,
							             sTmFields->tm_min,
							             sTmFields->tm_sec, NEWLINE );
 ErrorLogFileWriteMessage( " - Current process id .................: %ld%s", DbtuTools_CoreOsSystem_GetProcessId( 0 ), NEWLINE );
 ErrorLogFileWriteMessage( " - Source location ....................: %s (%ld)%s", pcCodeFileName,
                                                                     iCodeFileLine, NEWLINE );
 ErrorLogFileWriteMessage( "Important compiler macros:%s",           NEWLINE );
 ErrorLogFileWriteMessage( " - DBTUTOOLS_USEREXIT_DB2_INSTANCE ....: >%s<%s", DBTUTOOLS_USEREXIT_DB2_INSTANCE, NEWLINE );
 ErrorLogFileWriteMessage( " - DBTUTOOLS_USEREXIT_OPT_TREE_ROOT ...: >%s<%s", DBTUTOOLS_USEREXIT_OPT_TREE_ROOT, NEWLINE );
 ErrorLogFileWriteMessage( " - FILE_SEPARATOR .....................: %c%s",   FILE_SEPARATOR, NEWLINE );
 ErrorLogFileWriteMessage( "Parameters:%s",                          NEWLINE);
 for( I = 0; I < sGlobalStruct.iMainArgc; I++ )
 { ErrorLogFileWriteMessage( " - argv[%i] ............................: >%s<%s", I, ppcMainArgv[I], NEWLINE ); }
 ErrorLogFileWriteMessage( "-------------------------------------------------------------------------------%s", NEWLINE );
 /* ------------------------------------------------------------------------ */
}





/* ######################################################################### */
/* ##                                                                     ## */
/* ##   P A R S E A R G U M E N T S ()                                    ## */
/* ##                                                                     ## */
/* ######################################################################### */
void ParseArguments( void )
{
 /* ##### Presettings */
 /* ------------------------------------------------------------------------ */
 char **ppcMainArgv;
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -- */ 
 int    I;
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -- */
 ppcMainArgv = sGlobalStruct.pvMainArgv;
 /* ------------------------------------------------------------------------ */



 /* ##### "Prevent nested IF's"-Loop */
 /* ------------------------------------------------------------------------ */
 for(;;)
 {
  /* ##### Loop over all given arguments */
  /* ##### (0 = executable call, 1 = first argument,...) */
  /* ----------------------------------------------------------------------- */ 
  for( I = 1; I < sGlobalStruct.iMainArgc; I++ )
  {
   /* ##### Ensure, that the current argument has a minimum length */
   /* ---------------------------------------------------------------------- */
   if( STRLEN( ppcMainArgv[I] ) < 4 )
   {
    ERRORLOGFILEWRITEMESSAGEHEADER();
    ErrorLogFileWriteMessage( "===> ERROR: Parameter %i is smaller then the minimum argument length!%s", I, NEWLINE );
    ErrorLogFileWriteMessageFooter();
    sGlobalStruct.iProgRc = DBTUTOOLS_RC_USEREXIT_INVALID_PARAMETERS;
    break;
   }
   /* ---------------------------------------------------------------------- */
   
   /* ##### Process the current argument */
   /* ##### REMARK: The algorithm used within IBM's sample userexits is clever */
   /* #####         and more performant. But compared to the transaction log   */
   /* #####         processing that time saving does not count much.           */
   /* ---------------------------------------------------------------------- */
   /* ##### This are the arguments we are looking for */   
   if( StartsWith( ppcMainArgv[I], "-RQ" ) ) { sGlobalStruct.sArgsCommon.pcRequest  = &ppcMainArgv[I][3]; continue; }
   if( StartsWith( ppcMainArgv[I], "-DB" ) ) { sGlobalStruct.sArgsCommon.pcDbName   = &ppcMainArgv[I][3]; continue; }
   if( StartsWith( ppcMainArgv[I], "-NN" ) ) { sGlobalStruct.sArgsCommon.pcNodeName = &ppcMainArgv[I][3]; continue; }
   if( StartsWith( ppcMainArgv[I], "-LP" ) ) { sGlobalStruct.sArgsCommon.pcLogPath  = &ppcMainArgv[I][3]; continue; }
   if( StartsWith( ppcMainArgv[I], "-LN" ) ) { sGlobalStruct.sArgsCommon.pcLogName  = &ppcMainArgv[I][3]; continue; }
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -- */
   /* ##### This arguments will be ignored (they don't help use - yet) */
   if( StartsWith( ppcMainArgv[I], "-OS" ) ) { continue; }   /* ##### Operating system */
   if( StartsWith( ppcMainArgv[I], "-RL" ) ) { continue; }   /* ##### DB2 release */
   if( StartsWith( ppcMainArgv[I], "-AP" ) ) { continue; }   /* ##### ADSM password */
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -- */
   /* ##### This arguments are important, but we are not prepared for  */
   /* ##### TODO: Extend this userexit to support transaction logfiles */
   /* #####       which resists within raw devices. */
   if( ( StartsWith( ppcMainArgv[I], "-LS" ) ) ||   /* ##### Log file size */
       ( StartsWith( ppcMainArgv[I], "-SP" ) ) )    /* ##### Starting page */
   {
    ERRORLOGFILEWRITEMESSAGEHEADER();
    ErrorLogFileWriteMessage( "===> ERROR: The userexit has been called with some addition parameters (-LS%s", NEWLINE );
    ErrorLogFileWriteMessage( "            and/or -SP). This parameters indicates, that the transaction log%s", NEWLINE );
    ErrorLogFileWriteMessage( "            files resists within raw devices. The current implementation of%s", NEWLINE );
    ErrorLogFileWriteMessage( "            this userexit doesn't support this constellation.%s", NEWLINE );
    ErrorLogFileWriteMessage( "            Please check the DbtuTools project homepage for a newer version.%s", NEWLINE );    
    ErrorLogFileWriteMessageFooter();
    sGlobalStruct.iProgRc = DBTUTOOLS_RC_USEREXIT_SOFTWARE_FAILURE;
    break;   
   }
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -- */
   /* ##### On error,... */
   ERRORLOGFILEWRITEMESSAGEHEADER();
   ErrorLogFileWriteMessage( "===> ERROR: Parameter %i is unknown and unexpected. The userexit can not handle%s", I, NEWLINE );
   ErrorLogFileWriteMessage( "            it. Because it might be an important one, we stop processing here.%s", NEWLINE );
   ErrorLogFileWriteMessageFooter();
   sGlobalStruct.iProgRc = DBTUTOOLS_RC_USEREXIT_INVALID_PARAMETERS;
   break;   
   /* ---------------------------------------------------------------------- */
  }
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  /* ##### On error,... */
  if( sGlobalStruct.iProgRc != DBTUTOOLS_RC_USEREXIT_OK )
  { break; }
  /* ----------------------------------------------------------------------- */ 



  /* ##### Ensure, that all important arguments was given */
  /* ---------------------------------------------------------------------- */
  if( ( sGlobalStruct.sArgsCommon.pcRequest  == NULL ) ||
      ( sGlobalStruct.sArgsCommon.pcDbName   == NULL ) ||
      ( sGlobalStruct.sArgsCommon.pcNodeName == NULL ) ||
      ( sGlobalStruct.sArgsCommon.pcLogPath  == NULL ) ||
      ( sGlobalStruct.sArgsCommon.pcLogName  == NULL ) )
  {
   ERRORLOGFILEWRITEMESSAGEHEADER();
   ErrorLogFileWriteMessage( "===> ERROR: The userexit was called, but not all needed parameters was given.%s", NEWLINE );
   ErrorLogFileWriteMessage( "            Minimum parameters list is: -RQ, -DB, -NN, -LP, -LN%s", NEWLINE );
   ErrorLogFileWriteMessageFooter();
   sGlobalStruct.iProgRc = DBTUTOOLS_RC_USEREXIT_INVALID_PARAMETERS;
   break;
  }
  /* ---------------------------------------------------------------------- */



  /* ##### Leave "Prevent nested IF's"-Loop */
  break;
 }
 /* ----------------------------------------------------------------------- */
}





/* ######################################################################### */
/* ##                                                                     ## */
/* ##   P R O G R A M A C T I O N A R C H I V E ()                        ## */
/* ##                                                                     ## */
/* ######################################################################### */
void ProgramActionArchive( void )
{
 /* ##### Presettings */
 /* ------------------------------------------------------------------------ */
 char        acLogTempPath[DBTUTOOLS_MEMORY_SIZE_SMALL];
 struct stat sStatStruct;
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -- */
 int         iCallRc                                    = 0; 
 /* ------------------------------------------------------------------------ */



 /* ##### "Prevent nested IF's"-Loop */
 /* ------------------------------------------------------------------------ */
 for(;;)
 {
  /* ##### Assemble the logtemp path (location B) */
  /* ----------------------------------------------------------------------- */
  iCallRc = SNPRINTF( acLogTempPath, DBTUTOOLS_MEMORY_SIZE_SMALL, "%s%c%s%c%s%c%s%c%s%c%s", DBTUTOOLS_USEREXIT_OPT_TREE_ROOT, FILE_SEPARATOR,
 				                                  DBTUTOOLS_USEREXIT_DB2_INSTANCE, FILE_SEPARATOR,
				                                  "backup",                        FILE_SEPARATOR,
				                                  "logtemp",                       FILE_SEPARATOR,
					                          sGlobalStruct.sArgsCommon.pcDbName, FILE_SEPARATOR,
 				                         	  sGlobalStruct.sArgsCommon.pcNodeName );
  if( ( iCallRc == -1 ) || ( iCallRc >= DBTUTOOLS_MEMORY_SIZE_SMALL ) )
  {
   ERRORLOGFILEWRITEMESSAGEHEADER();
   ErrorLogFileWriteMessage( "===> ERROR: Not enough memory available for assembling the logtemp path.%s", NEWLINE );
   ErrorLogFileWriteMessage( "            iCallRc = %i%s", iCallRc, NEWLINE );
   ErrorLogFileWriteMessage( "            Please increase the amount of allocated memory within the source%s", NEWLINE );
   ErrorLogFileWriteMessage( "            code or open a bug report at the DbtuTools project homepage.%s",     NEWLINE );
   ErrorLogFileWriteMessageFooter();
   sGlobalStruct.iProgRc = DBTUTOOLS_RC_USEREXIT_SOFTWARE_FAILURE;
   break;
  }
  /* ----------------------------------------------------------------------- */



  /* ##### Ensure the existens of the logtemp path (location B) */
  /* ----------------------------------------------------------------------- */
  errno   = 0;
  iCallRc = stat( acLogTempPath, &sStatStruct );
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  /* ##### If there is something to do,... */
  if( !(( iCallRc == 0 ) && ( S_ISDIR( sStatStruct.st_mode ) )) )
  {
   /* ##### If logtemp path does not exists,... */
   if( (iCallRc != 0) && (errno == ENOENT) )
   {
printf("Not found!%s",NEWLINE);    
   }
   /* ##### Something other (wrong filetype,...) */
   else
   {
    ERRORLOGFILEWRITEMESSAGEHEADER();
    ErrorLogFileWriteMessage( "===> ERROR: Checking the status of the logtemp path failed%s", NEWLINE );
    ErrorLogFileWriteMessage( "            or returned an unexpected filetype.%s", NEWLINE );    
    ErrorLogFileWriteMessage( "            acLogTempPath = >%s<%s", acLogTempPath, NEWLINE );
    ErrorLogFileWriteMessage( "            iCallRc = %i%s",         iCallRc,       NEWLINE );
    ErrorLogFileWriteMessage( "            errno = %i%s",           errno,         NEWLINE );
    ErrorLogFileWriteMessageFooter();
    if( iCallRc == 0 )
    { sGlobalStruct.iProgRc = DBTUTOOLS_RC_USEREXIT_DBA_REQUIRED; }
    else
    { sGlobalStruct.iProgRc = DBTUTOOLS_RC_USEREXIT_ALLOC_FAILED; }
    break;    
   }
  }
  /* ----------------------------------------------------------------------- */
  
  
  


  printf("===> ARCHIVE <===\n");
  printf("%s\n", acLogTempPath);
  /* ===> stat() nutzen */

  /* ##### Leave "Prevent nested IF's"-Loop */
  break;
 }
 /* ----------------------------------------------------------------------- */





 /* ---------------------------------------------------------------------------
 /opt/dbtutools/INSTANCE/backup/logtemp/DB/NODE/S*.LOG
 /opt/dbtutools/INSTANCE/backup/logtemp/flagfiles/DB.NODE.S*.LOG
 /opt/dbtutools/INSTANCE/backup/logarch/DB/NODE/S*.LOG

 - Im Prinzip mu alles bis vor /DB/NODE oder DB.NODE da sein. Alles
   danach wird von L1 wenn notwendig angelegt (kein zeitliches Problem,
   da es hier nicht auf Zeit ankommt).
 - Wenn angelegt wird, dann dies aber in db2uext2.err rausschreiben.
 - Als Ownergruppe im Prinzip nichts zu tun, da S-Attribut gesetzt
   sein sollte. Dies ebenfalls nach fr die Verzeichnisse setzen.
 - Die oben genannten Verzeichnisse etc. per Defines in den Userexit
   reinreichen, aber nicht nach libdbtubase.* rein (dort auch mit
   der Umgebungsvariable arbeiten, nur der Userexit kann das nicht -
   zumindest nicht elegant).
 ------------------------------------------------------------------------------
 - Pro LoopZyklus (einzustellen per DbmCfg) liest der L2 die DbmCfg
   und prft ob er laufen soll oder gerade "unterdrckt wird". Ist
   dies der Fall, wird dies rausgeschrieben ins Log. Wenn nicht, dann
   liest er die Liste der flagfiles. Fr jede DB zu der es ein flagfile
   gibt wird die DbCfg gelesen und geprft ob diese gerade "unterdrckt"
   wird. Wenn ja dann erneut ins Log rausschreiben, ansonsten die Arbeit
   durchziehen.
 - Eine Idee whre, da nur L1 von Raw-Devices wissen mu. L2 kennt nur
   normale Files. Wenn in Raw-Devices recovered wird, dann stellt L2
   das File in ein Verzeichnis und L1 holt es von da und schreibt es
   in das Raw-Device (die Frage wie das mit Version 8 ist, ist dort
   vielleicht auch schon ein File vorgesehen?). Damit whre eigentlich
   der L2 sehr sauber, er wrde nur mit Dateien arbeiten.
 - ...
 --------------------------------------------------------------------------- */
}





/* ######################################################################### */
/* ##                                                                     ## */
/* ##   P R O G R A M A C T I O N R E T R I E V E ()                      ## */
/* ##                                                                     ## */
/* ######################################################################### */
void ProgramActionRetrieve( void )
{ printf("===> RETRIEVE <===\n"); }





/* ######################################################################### */
/* ##                                                                     ## */
/* ##   S I G N A L H A N D L E R ()                                      ## */
/* ##                                                                     ## */
/* ######################################################################### */
void SignalHandler( int iSignalNumber )
{
 /* ##### Presettings */
 /* ------------------------------------------------------------------------ */
 char acSignalName[32];
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -- */
 sGlobalStruct.iProgRc = DBTUTOOLS_RC_USEREXIT_PROGRAM_INTERRUPTED;
 /* ------------------------------------------------------------------------ */ 



 /* ##### Inform the administrator by writing to the error log file */
 /* ------------------------------------------------------------------------ */
 switch( iSignalNumber )
 {
#if defined(DBTUTOOLS_OS_PLATFORM_UNIX)
  case SIGHUP:  sprintf( acSignalName, "SIGHUP" );    break;
#endif
  case SIGINT:  sprintf( acSignalName, "SIGINT" );    break;
  case SIGABRT: sprintf( acSignalName, "SIGABRT" );   break;
  case SIGTERM: sprintf( acSignalName, "SIGTERM" );   break;
#if defined(DBTUTOOLS_OS_PLATFORM_UNIX)
  case SIGUSR1: sprintf( acSignalName, "SIGUSR1" );   break;
#endif
  default:      sprintf( acSignalName, "<unknown>" ); break;
 }
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -- */
 ERRORLOGFILEWRITEMESSAGEHEADER();
 ErrorLogFileWriteMessage( "===> ERROR: Signal received. The program will be terminated!%s", NEWLINE );
 ErrorLogFileWriteMessage( "     SignalNumber = %i (%s)%s", iSignalNumber, acSignalName, NEWLINE );
 ErrorLogFileWriteMessageFooter();
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -- */ 
 ErrorLogFileClose();
 /* ------------------------------------------------------------------------ */



 /* ##### End the program */
 /* ------------------------------------------------------------------------ */ 
 exit( sGlobalStruct.iProgRc );
 /* ------------------------------------------------------------------------ */ 
}




