/* ######################################################################### */
/* ##                                                                     ## */
/* ##  D B T U T O O L S / L I B D B T U B A S E . H                      ## */
/* ##                                                                     ## */
/* ## ------------------------------------------------------------------- ## */
/* ##                                                                     ## */
/* ##  Job .......: Implements basis defines / functions.                 ## */
/* ##                                                                     ## */
/* ## ------------------------------------------------------------------- ## */
/* ##                                                                     ## */
/* ##  Copyright (C) 2002  Daniel Scheibli                                ## */
/* ##                                                                     ## */
/* ##  This library is free software; you can redistribute it and/or      ## */
/* ##  modify it under the terms of the GNU Lesser General Public         ## */
/* ##  License as published by the Free Software Foundation; either       ## */
/* ##  version 2.1 of the License, or (at your option) any later version. ## */
/* ##                                                                     ## */
/* ##  This library is distributed in the hope that it will be useful,    ## */
/* ##  but WITHOUT ANY WARRANTY; without even the implied warranty of     ## */
/* ##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU  ## */
/* ##  Lesser General Public License for more details.                    ## */
/* ##                                                                     ## */
/* ##  You should have received a copy of the GNU Lesser General Public   ## */
/* ##  License along with this library; if not, write to the Free         ## */
/* ##  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,     ## */
/* ##  MA 02111-1307 USA                                                  ## */
/* ##                                                                     ## */
/* ## ------------------------------------------------------------------- ## */
/* ##                                                                     ## */
/* ##  Author ....: Daniel Scheibli <daniel.scheibli@edelbyte.org>        ## */
/* ##  Date ......: 2002-05-12                                            ## */
/* ##  Changes ...: ----- libdbtubase.h V00.99/R020 --------------------- ## */
/* ##               2003-01-03 - Added platform detection via macro to    ## */
/* ##                            define environmental settings.           ## */
/* ##               2003-01-02 - Changed a lot of defines and functions   ## */
/* ##                            from Dbtu_* to DbtuTools_*.              ## */
/* ##               ----- libdbtubase.h V00.99/R011 --------------------- ## */
/* ##               <none>                                                ## */
/* ##               ----- libdbtubase.h V00.99/R010 --------------------- ## */
/* ##               2002-06-27 - Added implementation for some new        ## */
/* ##                            element types to Dbtu_Converter_...      ## */
/* ##                            ...SnapshotBufferElement2String();       ## */
/* ##               2002-07-27 - Added Dbtu_System_GetEndianType().       ## */
/* ##               ----- libdbtubase.h V00.99/R003 --------------------- ## */
/* ##               2002-06-07 - Added implementation for some new        ## */
/* ##                            element types to Dbtu_Converter_...      ## */
/* ##                            ...SnapshotBufferElement2String();       ## */
/* ##               ----- libdbtubase.h V00.99/R002 --------------------- ## */
/* ##               <none>                                                ## */
/* ##                                                                     ## */
/* ######################################################################### */
#ifndef ___ORG_EDELBYTE_DBTUTOOLS_LIBDBTUBASE_H___
#define ___ORG_EDELBYTE_DBTUTOOLS_LIBDBTUBASE_H___



/* ##### Presettings (include other headerfiles) */
/* ------------------------------------------------------------------------- */
#include<stdio.h>
#include<stdlib.h>
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
#include<ctype.h>
#include<errno.h>
#include<signal.h>
#include<stdarg.h>
#include<string.h>
#include<sys/stat.h>
#include<sys/types.h>
#include<time.h>
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
#if defined(DBTUTOOLS_OS_PLATFORM_UNIX)
 #include<unistd.h>
#endif
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
#if defined(DBTUTOOLS_OS_PLATFORM_WINTEL)
 #include<windows.h>
#endif
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
#include<db2ApiDf.h>
#include<sqlenv.h>
#include<sqlmon.h>
#include<sqlutil.h>
/* ------------------------------------------------------------------------- */





/* ##### Presettings (generic defines) */
/* ------------------------------------------------------------------------- */
#if defined(DBTUTOOLS_OS_PLATFORM_UNIX) && defined(DBTUTOOLS_OS_PLATFORM_WINTEL)
 #error ===> ERROR: Only one DBTUTOOLS_OS_PLATFORM_* macro is allowed!
#endif
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
#if !defined(DBTUTOOLS_OS_PLATFORM_UNIX) && !defined(DBTUTOOLS_OS_PLATFORM_WINTEL)
 #error ===> ERROR: You must define one of the DBTUTOOLS_OS_PLATFORM_* macro!
#endif
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
#ifndef TRUE
 #define TRUE    (1==1)
#endif
#ifndef FALSE
 #define FALSE   (1==0)
#endif
/* ------------------------------------------------------------------------- */





/* ##### Presettings (environmental settings) */
/* ------------------------------------------------------------------------- */
#if defined(DBTUTOOLS_OS_PLATFORM_UNIX)
 #ifndef NEWLINE
  #define NEWLINE          "\n"
 #endif
 #ifndef FILE_SEPARATOR
  #define FILE_SEPARATOR   '/'
 #endif
#endif
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
#if defined(DBTUTOOLS_OS_PLATFORM_WINTEL)
 #ifndef NEWLINE
  #define NEWLINE          "\r\n"
 #endif
 #ifndef FILE_SEPARATOR
  #define FILE_SEPARATOR   '\\'
 #endif
#endif
/* ------------------------------------------------------------------------- */
#ifndef EOS
 #define EOS     '\0'
#endif
#ifndef SPACE
 #define SPACE   ' '
#endif
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
#ifndef STDOUT
 #define STDOUT   stdout
#endif
#ifndef STDDBG
 #define STDDBG   stderr
#endif
#ifndef STDERR
 #define STDERR   stderr
#endif
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
#ifndef S_ISDIR
 #define S_ISDIR(m)   ( ( (m) & S_IFMT ) == S_IFDIR )
#endif
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
#ifndef ISALHPANUM
 #define ISALPHANUM   isalnum
#endif
#ifndef ISHEXDIGIT
 #define ISHEXDIGIT   isxdigit
#endif
#ifndef SNPRINTF
 #if defined(DBTUTOOLS_OS_PLATFORM_UNIX)
  #define SNPRINTF    snprintf
 #endif
 #if defined(DBTUTOOLS_OS_PLATFORM_WINTEL)
  #define SNPRINTF    _snprintf
 #endif
#endif
#ifndef STRCMP
 #define STRCMP       strcmp
#endif
#ifndef STRCPY
 #define STRCPY       strcpy
#endif
#ifndef STRICMP
 #define STRICMP      strcasecmp
#endif
#ifndef STRLEN
 #define STRLEN       strlen
#endif
#ifndef STRNCMP
 #define STRNCMP      strncmp
#endif
#ifndef STRNCPY
 #define STRNCPY      strncpy
#endif
#ifndef STRSTR
 #define STRSTR       strstr
#endif
#ifndef STRTOL
 #define STRTOL       strtol
#endif
/* ------------------------------------------------------------------------- */





/* ##### Presettings (DB2 specific things) */
/* ------------------------------------------------------------------------- */
#define DBTUTOOLS_VERSION_MONITOR_USED   SQLM_DBMON_VERSION7
#define DBTUTOOLS_VERSION_STRUCT_USED    db2Version710
/* ------------------------------------------------------------------------- */
#define DBTUTOOLS_LIMIT_LENGTH_DB_NAME   8
#define DBTUTOOLS_LIMIT_LENGTH_TS_NAME   18
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
#define DBTUTOOLS_MEMORY_SIZE_PAGE       4096
#define DBTUTOOLS_MEMORY_SIZE_SMALL      256
#define DBTUTOOLS_MEMORY_SIZE_MEDIUM     4096
#define DBTUTOOLS_MEMORY_SIZE_LARGE      32768
/* ------------------------------------------------------------------------- */
#define DBTUTOOLS_FORMAT_OUTPUT_DEFAULT           0
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
#define DBTUTOOLS_FORMAT_OUTPUT_NUMDEC            100
#define DBTUTOOLS_FORMAT_OUTPUT_NUMHEX            101
#define DBTUTOOLS_FORMAT_OUTPUT_STRING            102
#define DBTUTOOLS_FORMAT_OUTPUT_STRINGORHYPEN     103
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
#define DBTUTOOLS_FORMAT_OUTPUT_APPLSTATUS        200
#define DBTUTOOLS_FORMAT_OUTPUT_APPLIDLETIME      201
#define DBTUTOOLS_FORMAT_OUTPUT_APPLID_TCPIP      206
#define DBTUTOOLS_FORMAT_OUTPUT_LOCKOBJECTTYPE    202
#define DBTUTOOLS_FORMAT_OUTPUT_LOCKMODE          203
#define DBTUTOOLS_FORMAT_OUTPUT_LOCKMODEACRONYM   204
#define DBTUTOOLS_FORMAT_OUTPUT_LOCKSTATUS        205
/* ------------------------------------------------------------------------- */
#define DBTUTOOLS_RC_COMMON_OK        0
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
#define DBTUTOOLS_RC_COMMON_WARNING   2
#define DBTUTOOLS_RC_COMMON_ERROR     4
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* ##### The return codes - listed below - will be used within the db2uext2  */
/* ##### implementation of the DbtuTools. The values and there meaning are   */
/* ##### based on the values which used within sample userexit provided by   */
/* ##### IBM (see $DB2DIR/samples/c/db2uext2.c*).                            */
#define DBTUTOOLS_RC_USEREXIT_OK                    0    /* ##### Escalation Level: none, everything is fine */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
#define DBTUTOOLS_RC_USEREXIT_ALLOC_FAILED          4    /* ##### Escalation Level: single request */
#define DBTUTOOLS_RC_USEREXIT_DBA_REQUIRED          8    /* ##### Escalation Level: single request */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
#define DBTUTOOLS_RC_USEREXIT_HARDWARE_FAILURE      12   /* ##### Escalation Level: all requests (for the same database) */
#define DBTUTOOLS_RC_USEREXIT_SOFTWARE_FAILURE      16   /* ##### Escalation Level: all requests (for the same database) */
#define DBTUTOOLS_RC_USEREXIT_INVALID_PARAMETERS    20   /* ##### Escalation Level: all requests (for the same database) */
#define DBTUTOOLS_RC_USEREXIT_UNKNOWN_ERRROR        28   /* ##### Escalation Level: all requests (for the same database) */
#define DBTUTOOLS_RC_USEREXIT_PROGRAM_INTERRUPTED   32   /* ##### Escalation Level: all requests (for the same database) */
/* ------------------------------------------------------------------------- */





/* ##### Presettings (function prototypes) */
/* ------------------------------------------------------------------------- */
/* libdbtubase.c */
extern int    DbtuTools_ApiWrapper_MonitorSwitches( struct sqlm_recording_group *pasMonSwitch );
extern int    DbtuTools_ApiWrapper_GetSnapshot( struct sqlma  *psSqlMaStruct,
                                                void         **ppvStreamBuffer );
extern char  *DbtuTools_Converter_ConsistencyToken2Timestamp( char *pcConsistencyToken );
extern int    DbtuTools_Converter_SnapshotBufferElement2String( struct sqlm_header_data *psBufferElement,
                                                                int                      iOutputFormat,
                                                                char                    *pcStringBuffer,
							        int                      iMaxStringBufferSize );
extern int    DbtuTools_Parser_FindSnapshotBufferElement( sqluint16  uiElement,
                                                          void      *pvStreamBuffer,
						          void     **ppvMatchingElementsBuffer,
					                  int        iMaxDepth );
extern int    DbtuTools_CoreOsSystem_GetEndianType( void );
extern long   DbtuTools_CoreOsSystem_GetProcessId( unsigned int uiLevel );
#if defined(DBTUTOOLS_OS_PLATFORM_UNIX)
 extern long  DbtuTools_CoreOsSystem_GetProcessId_Unix( unsigned int uiLevel );
#endif
#if defined(DBTUTOOLS_OS_PLATFORM_WINTEL)
 extern long  DbtuTools_CoreOsSystem_GetProcessId_Wintel( unsigned int uiLevel );
#endif
extern void   DbtuToolsSqlAbend( struct sqlca *SqlCaPointer );
extern int    StartsWith( char *pcHayStack, char *pcNeedle );
extern char  *Strip( char *pcString );
/* ------------------------------------------------------------------------- */



#endif




